package sun.util.calendar;

import java.util.TimeZone;

public abstract class AbstractCalendar extends CalendarSystem
{
    static final int SECOND_IN_MILLIS = 1000;
    static final int MINUTE_IN_MILLIS = 60000;
    static final int HOUR_IN_MILLIS = 3600000;
    static final int DAY_IN_MILLIS = 86400000;
    static final int EPOCH_OFFSET = 719163;
    private Era[] eras;
    
    protected AbstractCalendar() {
    }
    
    @Override
    public Era getEra(String s) {
        if (this.eras != null) {
            int i = 0;
            while (i < this.eras.length) {
                if (this.eras[i].equals(s)) {
                    return this.eras[i];
                }
                ++i;
            }
        }
        return null;
    }
    
    @Override
    public Era[] getEras() {
        Era[] array = null;
        if (this.eras != null) {
            array = new Era[this.eras.length];
            System.arraycopy(this.eras, 0, array, 0, this.eras.length);
            return array;
        }
        return array;
    }
    
    @Override
    public void setEra(CalendarDate calendarDate, String s) {
        if (this.eras == null) {
            return;
        }
        int i = 0;
        while (i < this.eras.length) {
            Era era = this.eras[i];
            if (era != null && era.getName().equals(s)) {
                calendarDate.setEra(era);
                return;
            }
            ++i;
        }
        throw new IllegalArgumentException("unknown era name: " + s);
    }
    
    protected void setEras(Era[] eras) {
        this.eras = eras;
    }
    
    @Override
    public CalendarDate getCalendarDate() {
        return this.getCalendarDate(System.currentTimeMillis(), this.newCalendarDate());
    }
    
    @Override
    public CalendarDate getCalendarDate(long n) {
        return this.getCalendarDate(n, this.newCalendarDate());
    }
    
    @Override
    public CalendarDate getCalendarDate(long n, TimeZone timeZone) {
        return this.getCalendarDate(n, this.newCalendarDate(timeZone));
    }
    
    @Override
    public CalendarDate getCalendarDate(long date, CalendarDate calendarDate) {
        int n = 0;
        int zoneOffset = 0;
        int daylightSaving = 0;
        long n2 = 0L;
        TimeZone zone = calendarDate.getZone();
        if (zone != null) {
            int[] array = new int[2];
            if ((zone instanceof ZoneInfo)) {
                zoneOffset = ((ZoneInfo)zone).getOffsets(date, array);
            }
            else {
                zoneOffset = zone.getOffset(date);
                array[0] = zone.getRawOffset();
                array[1] = zoneOffset - array[0];
            }
            n2 = (long)(zoneOffset / 86400000);
            n = zoneOffset % 86400000;
            daylightSaving = array[1];
        }
        calendarDate.setZoneOffset(zoneOffset);
        calendarDate.setDaylightSaving(daylightSaving);
        long n3 = n2 + date / 86400000L;
        int i = n + (int)(date % 86400000L);
        if (i >= 86400000) {
            i -= 86400000;
            ++n3;
        }
        else {
            while (i < 0) {
                i += 86400000;
                --n3;
            }
        }
        this.getCalendarDateFromFixedDate(calendarDate, n3 + 719163L);
        this.setTimeOfDay(calendarDate, i);
        calendarDate.setLeapYear(this.isLeapYear(calendarDate));
        calendarDate.setNormalized(true);
        return calendarDate;
    }
    
    @Override
    public long getTime(CalendarDate calendarDate) {
        long n = (this.getFixedDate(calendarDate) - 719163L) * 86400000L + this.getTimeOfDay(calendarDate);
        int n2 = 0;
        TimeZone zone = calendarDate.getZone();
        if (zone != null) {
            if (calendarDate.isNormalized()) {
                return n - (long)calendarDate.getZoneOffset();
            }
            int[] array = new int[2];
            if (calendarDate.isStandardTime()) {
                if ((zone instanceof ZoneInfo)) {
                    ((ZoneInfo)zone).getOffsetsByStandard(n, array);
                    n2 = array[0];
                }
                else {
                    n2 = zone.getOffset(n - (long)zone.getRawOffset());
                }
            }
            else if ((zone instanceof ZoneInfo)) {
                n2 = ((ZoneInfo)zone).getOffsetsByWall(n, array);
            }
            else {
                n2 = zone.getOffset(n - (long)zone.getRawOffset());
            }
        }
        long n3 = n - (long)n2;
        this.getCalendarDate(n3, calendarDate);
        return n3;
    }
    
    protected long getTimeOfDay(CalendarDate calendarDate) {
        long timeOfDay = calendarDate.getTimeOfDay();
        if (timeOfDay != Long.MIN_VALUE) {
            return timeOfDay;
        }
        long timeOfDayValue = this.getTimeOfDayValue(calendarDate);
        calendarDate.setTimeOfDay(timeOfDayValue);
        return timeOfDayValue;
    }
    
    public long getTimeOfDayValue(CalendarDate calendarDate) {
        return (((long)calendarDate.getHours() * 60L + (long)calendarDate.getMinutes()) * 60L + (long)calendarDate.getSeconds()) * 1000L + (long)calendarDate.getMillis();
    }
    
    @Override
    public CalendarDate setTimeOfDay(CalendarDate calendarDate, int n) {
        if (n < 0) {
            throw new IllegalArgumentException();
        }
        int normalized = calendarDate.isNormalized();
        int hours = n / 3600000;
        int n2 = n % 3600000;
        int minutes = n2 / 60000;
        int n3 = n2 % 60000;
        int seconds = n3 / 1000;
        int millis = n3 % 1000;
        calendarDate.setHours(hours);
        calendarDate.setMinutes(minutes);
        calendarDate.setSeconds(seconds);
        calendarDate.setMillis(millis);
        calendarDate.setTimeOfDay((long)n);
        if (hours < 24 && normalized) {
            calendarDate.setNormalized((boolean)normalized);
            return calendarDate;
        }
        return calendarDate;
    }
    
    @Override
    public int getWeekLength() {
        return 7;
    }
    
    protected abstract boolean isLeapYear(CalendarDate p0);
    
    @Override
    public CalendarDate getNthDayOfWeek(int n, int n2, CalendarDate calendarDate) {
        CalendarDate calendarDate2 = (CalendarDate)calendarDate.clone();
        this.normalize(calendarDate2);
        long fixedDate = this.getFixedDate(calendarDate2);
        long n3;
        if (n > 0) {
            n3 = (long)(7 * n) + getDayOfWeekDateBefore(fixedDate, n2);
        }
        else {
            n3 = (long)(7 * n) + getDayOfWeekDateAfter(fixedDate, n2);
        }
        this.getCalendarDateFromFixedDate(calendarDate2, n3);
        return calendarDate2;
    }
    
    static long getDayOfWeekDateBefore(long n, int n2) {
        return getDayOfWeekDateOnOrBefore(n - 1L, n2);
    }
    
    static long getDayOfWeekDateAfter(long n, int n2) {
        return getDayOfWeekDateOnOrBefore(n + 7L, n2);
    }
    
    public static long getDayOfWeekDateOnOrBefore(long n, int n2) {
        long n3 = n - (long)(n2 - 1);
        if (n3 >= 0L) {
            return n - n3 % 7L;
        }
        return n - CalendarUtils.mod(n3, 7L);
    }
    
    protected abstract long getFixedDate(CalendarDate p0);
    
    protected abstract void getCalendarDateFromFixedDate(CalendarDate p0, long p1);
    
    public boolean validateTime(CalendarDate calendarDate) {
        int hours = calendarDate.getHours();
        if (hours < 0 || hours >= 24) {
            return false;
        }
        int minutes = calendarDate.getMinutes();
        if (minutes < 0 || minutes >= 60) {
            return false;
        }
        int seconds = calendarDate.getSeconds();
        if (seconds < 0 || seconds >= 60) {
            return false;
        }
        int millis = calendarDate.getMillis();
        return millis >= 0 && millis < 1000;
    }
    
    int normalizeTime(CalendarDate calendarDate) {
        long timeOfDay = this.getTimeOfDay(calendarDate);
        long floorDivide = 0L;
        if (timeOfDay >= 86400000L) {
            floorDivide = timeOfDay / 86400000L;
            timeOfDay %= 86400000L;
        }
        else if (timeOfDay < 0L) {
            floorDivide = CalendarUtils.floorDivide(timeOfDay, 86400000L);
            if (floorDivide != 0L) {
                timeOfDay -= 86400000L * floorDivide;
            }
        }
        if (floorDivide != 0L) {
            calendarDate.setTimeOfDay(timeOfDay);
        }
        calendarDate.setMillis((int)(timeOfDay % 1000L));
        long n = timeOfDay / 1000L;
        calendarDate.setSeconds((int)(n % 60L));
        long n2 = n / 60L;
        calendarDate.setMinutes((int)(n2 % 60L));
        calendarDate.setHours((int)(n2 / 60L));
        return (int)floorDivide;
    }
}
